// VinciaRoot.h is a part of the VINCIA plug-in to the PYTHIA 8 generator.
// Copyright (C) 2010 W.T. Giele, D.A. Kosower, P.Z. Skands.
// VINCIA is licenced under the GNU GPL version 2.
// Please respect the MCnet Guidelines for Event Generator Authors and Users, 
//               http://www.montecarlonet.org/GUIDELINES

// This file contains header information for the Vincia ROOT utilities

#ifndef VINCIAROOT_H
#define VINCIAROOT_H

// Include specific ROOT headers 

// Main ROOT header
#include "TROOT.h"
// 1D and 2D histograms
#include "TH1.h"
#include "TH2.h"
#include "TGraphAsymmErrors.h"
#include "TPaveStats.h"
#include "TLine.h"
// Widget windows
#include "TCanvas.h"
#include "TApplication.h"
// To read/write files
#include "TDirectory.h"
#include "TFile.h"

// Include Pythia 8 headers and name space
#include "Basics.h"
#include "PythiaStdlib.h"

using namespace Pythia8;

// Put all the rest in Vincia namespace

namespace Vincia {
  
  class VinciaRoot {

  public:

    // Constructor
    VinciaRoot() {
      // Switch off display by default
      useDisplay = false;
      appIsInit  = false;
      // Switch on storage of stat uncertainties 
      TH1::SetDefaultSumw2();
      // Default verbose level
      verbose = 1;
    }
    
    // Initialize Displays
    bool createDisplay(string nameIn="VINCIA", int xSizIn=1024, int ySizIn=800, int xOffsetIn=-1,int yOffsetIn=-1) {
      cout<<"createDisplay..."<<endl;
      displayName.push_back(nameIn);
      xSiz.push_back(xSizIn);
      ySiz.push_back(ySizIn);      
      if (xOffsetIn == -1) xOffsetIn = 16*xOffset.size();
      if (yOffsetIn == -1) yOffsetIn = 16*yOffset.size();
      xOffset.push_back(xOffsetIn);
      yOffset.push_back(yOffsetIn);      
      nColsSav.push_back(0);
      nRowsSav.push_back(0);
      if (! appIsInit) {
	cout<<"  initializing app"<<endl;
	useDisplay=true;
	gROOT->Reset();
	gROOT->SetStyle("Plain");	
	TApplication *vcApp = new TApplication("vcApp",0,0);
	vcApp->InitializeGraphics();
	appIsInit = true;
      }      
      cout<<"  adding display"<<endl;
      displays.push_back(new TCanvas(nameIn.c_str(),nameIn.c_str(),
				     xOffsetIn,       yOffsetIn,
				     xOffsetIn+xSizIn,yOffsetIn+ySizIn));
      displays[displays.size()-1]->Modified();
      displays[displays.size()-1]->Update();
      cout<<"  returning";
      return true;
    }
    
    // Read file containing data and store points
    bool addDataFile(string dataFile, bool normalize=true) {

      // Define read-in stream
      ifstream in;

      // Return false if fails
      in.open(dataFile.c_str());
      if (! in) return false;

      // Arrays for data storage
      Float_t xc,xlo,xhi,yc,sig1,sig2,sig3,sig4;
      Int_t nlines = 0;
      Float_t x[200],exL[200],exR[200],xT[200],exLT[200],exRT[200];
      Float_t y[200],eyStatL[200],eyStatH[200],eyTotL[200],eyTotH[200], zero[200];
      Float_t yT[200],eyStatLT[200],eyStatHT[200],eyTotLT[200],eyTotHT[200];
      Float_t sumsig=0.0;
      
      // Set data format depending on which file we're reading
      int    dataFormat = 0;
      double normFactor = 1.0;
      bool normalizeData = normalize;
      // Format  0: x, y, Err
      //           with bin widths fixed to xc +/- 0.5
      // Format  1: x, y, sigStat, sigSys
      //           with bin widths fixed to xc +/- 1 
      // Format  2: xlo, xhi, y, sigStatP, sigStatM, sigSysP, sigSysM
      //           with bin widths fixed to xc +/- 1 
      // Format  3: xlo, xhi, y, sigStatP, sigStatM, sigSysP, sigSysM
      //           with dynamic bin widths (bins assumed symmetric)
      // Format  4: xlo, xhi, y, sigStatPM, sigSysP, sigSysM
      //           with dynamic bin widths (bins assumed symmetric)
      // Format  5: xlo, xhi, y, sigStatPM, sigSys1PM, sigSys2PM
      // Format  6: xlo, xhi, y, sigTotPM, sigStatPM, sigSysPM
      // Format  7: xlo, xhi, y, sigStat, sigSys
      // Format -7: exp(xlo), exp(xhi), y, sigStat, sigSys

      if (dataFile.find("L3-91-Nch-udsc") != string::npos) {
	dataFormat = 2;
      } else if (dataFile.find("LEP91-Oblateness") != string::npos) {
	dataFormat = 4;
      } else if (dataFile.find("LEP91-Major") != string::npos) {
	dataFormat = 4;
      } else if (dataFile.find("Delphi") != string::npos) {
	dataFormat = 3;
      } else if (dataFile.find("Aleph") != string::npos) {
	dataFormat = 3;
	if (dataFile.find("xK") != string::npos) {
	  dataFormat = -7;
	  normalizeData = false;
	}
      } else if (dataFile.find("LEP91-Minor") != string::npos) {
	dataFormat = 4;
      } else if (dataFile.find("L3") != string::npos) {
	dataFormat = 3;
      } else if (dataFile.find("LnY") != string::npos) {
	dataFormat = 5;
      }
	
      // Read next data file line
      while (1) {
	if (dataFormat == 0 || dataFormat == 1) {
	  if (dataFormat == 0) {
	    sig2 = 0.0;
	    in >> xc >> yc >> sig1; 
	  } else if (dataFormat == 1) {
	    in >> xc >> yc >> sig1 >> sig2;	  
	  }
	  if (!in.good()) break;
	  // Process x value
	  x[nlines] = xc;
	  if (nlines == 0) {
	    exL[nlines] = 1;
	    exR[nlines] = 1;
	    if (dataFormat == 0) exL[nlines] = 0.5;
	    if (dataFormat == 0) exR[nlines] = 0.5;
	  } else {
	    exL[nlines] = xc - (x[nlines-1]+exR[nlines-1]);
	    exR[nlines] = exL[nlines];
	  }
	  // Compute integral 
	  sumsig += yc;
	  // Process y value
	  y[nlines] = yc;
	  eyStatL[nlines] = sig1;
	  eyStatH[nlines] = sig1;
	  eyTotL[nlines] = sig1+sig2;
	  eyTotH[nlines] = sig1+sig2;
	} 
	// Format 2
	else if (dataFormat == 2) {
	  in >> xlo >> xhi >> yc >> sig1 >> sig2 >> sig3 >> sig4;
	  if (!in.good()) break;
	  // Take abs values
	  sig1 = abs(sig1);
	  sig2 = abs(sig2);
	  sig3 = abs(sig3);
	  sig4 = abs(sig4);
	  // Process x value
	  x[nlines] = (xlo+xhi)/2.;
	  exL[nlines] = 1;
	  exR[nlines] = 1;
	  xlo = x[nlines] - exL[nlines];
	  xhi = x[nlines] + exR[nlines];
	  // Process y value
	  y[nlines] = yc;
	  eyStatH[nlines] = sig1;
	  eyStatL[nlines] = sig2;
	  // Add stat and sys linearly
	  eyTotH[nlines] = sig1 + sig3;
	  eyTotL[nlines] = sig2 + sig4;
	  sumsig += yc;
	}
	// Format 3
	else if (dataFormat == 3 || dataFormat == 4 || abs(dataFormat) == 7) {
	  if (dataFormat == 3) {
	    in >> xlo >> xhi >> yc >> sig1 >> sig2 >> sig3 >> sig4;
	  } else if (dataFormat == 4) {
	    in >> xlo >> xhi >> yc >> sig1 >> sig3 >> sig4;
	  } else if (abs(dataFormat) == 7) {
	    in >> xlo >> xhi >> yc >> sig1 >> sig3;
	  }
	  if (!in.good()) break;
	  if (dataFormat == 4) sig2 = sig1;
	  if (abs(dataFormat) == 7) {
	    sig2 = sig1;
	    sig4 = sig3;
	  }
	  if (dataFormat == -7) {
	    xlo  = log(xlo);
	    xhi  = log(xhi);
	  }
	  // Take abs values
	  sig1 = abs(sig1);
	  sig2 = abs(sig2);
	  sig3 = abs(sig3);
	  sig4 = abs(sig4);
	  // Process x value
	  x[nlines] = (xlo+xhi)/2.;
	  exL[nlines] = (xhi-xlo)/2.;
	  exR[nlines] = (xhi-xlo)/2.;
	  xlo = x[nlines] - exL[nlines];
	  xhi = x[nlines] + exR[nlines];
	  // Process y value
	  y[nlines] = yc;
	  eyStatH[nlines] = sig1;
	  eyStatL[nlines] = sig2;
	  // Add stat and sys linearly
	  eyTotH[nlines] = sig1 + sig3;
	  eyTotL[nlines] = sig2 + sig4;
	  sumsig += yc * (xhi - xlo);
	  //	  cout << dataFile<<" : "<< xlo << " "<<xhi<<endl;
	}
	// Format 5
	else if (dataFormat == 5 || dataFormat == 6) {
	  in >> xlo >> xhi >> yc >> sig1 >> sig2 >> sig3;
	  if (!in.good()) break;
	  // Take abs values
	  sig1 = abs(sig1);
	  sig2 = abs(sig2);
	  sig3 = abs(sig3);
	  // Process x value
	  x[nlines] = (xlo+xhi)/2.;
	  exL[nlines] = (xhi-xlo)/2.;
	  exR[nlines] = (xhi-xlo)/2.;
	  xlo = x[nlines] - exL[nlines];
	  xhi = x[nlines] + exR[nlines];
	  // Process y value
	  y[nlines] = yc;
	  if (dataFormat == 5) {
	    eyStatH[nlines] = sig1;
	    eyStatL[nlines] = sig2;
	  } else {
	    eyStatH[nlines] = sig2;
	    eyStatL[nlines] = sig2;
	  }
	  // Add stat and sys linearly
	  if (dataFormat == 5) {
	    eyTotH[nlines] = sig1 + sig2 + sig3;
	    eyTotL[nlines] = sig2 + sig2 + sig3;
	  } else {
	    eyTotH[nlines] = sig1;
	    eyTotL[nlines] = sig1;
	  }
	  sumsig += yc * (xhi - xlo);
	  //	  cout << dataFile<<" : "<< xlo << " "<<xhi<<endl;
	}
	nlines++;
	//	if (nlines < 200) printf("xlo=%8f, xhi=%8f, y=%8f, ey=+%8f-%8f\n",xlo,xhi,yc,abs(sig1)+abs(sig3),abs(sig2)+abs(sig4));
      }
      if (verbose >= 1) {
	printOut("VinciaRoot",num2str(nlines)+" readable data points in "+dataFile);
      }      

      // Normalize?
      if (normFactor != 1.0) {
	for (int i=0; i<200; i++) {
	  y[i] *= normFactor;
	  eyTotL[i] *= normFactor;
	  eyTotH[i] *= normFactor;
	  eyStatL[i] *= normFactor;
	  eyStatH[i] *= normFactor;
	}
      }

      if (normalizeData && sumsig >= 0.0) {
	for (int i=0; i<200; i++) {
	  y[i] /= sumsig;
	  eyTotL[i] /= sumsig;
	  eyTotH[i] /= sumsig;
	  eyStatL[i] /= sumsig;
	  eyStatH[i] /= sumsig;
	}
      }

      // Special case for L3 Thrust measurement: T -> 1-T
      if (dataFile == "data/L3-91-Thrust-udsc.vec") {
	// Invert
	for (int i=0; i<nlines; i++) {
	  xT[i]       = 1-x[nlines-i-1];
	  exLT[i]     = exR[nlines-i-1];
	  exRT[i]     = exL[nlines-i-1];
	  yT[i]       = y[nlines-i-1];
	  eyTotLT[i]  = eyTotL[nlines-i-1];
	  eyTotHT[i]  = eyTotH[nlines-i-1];
	  eyStatLT[i] = eyStatL[nlines-i-1];
	  eyStatHT[i] = eyStatH[nlines-i-1];
	}
	// Save 
	for (int i=0; i<nlines; i++) {
	  x[i]        = xT[i];
	  exL[i]      = exRT[i];
	  exR[i]      = exLT[i];
	  y[i]        = yT[i];
	  eyTotL[i]   = eyTotLT[i];
	  eyTotH[i]   = eyTotHT[i];
	  eyStatL[i]  = eyStatLT[i];
	  eyStatH[i]  = eyStatHT[i];
	}
      }

      TGraphAsymmErrors* grTot = 
	new TGraphAsymmErrors(nlines,x,y,exL,exR,eyTotL,eyTotH);
      grTot->SetMarkerColor(1);
      grTot->SetMarkerStyle(21);
      grTot->SetLineColor(1);
      grTot->SetMarkerSize(0.6);
      
      TGraphAsymmErrors* grStat = 
	new TGraphAsymmErrors(nlines,x,y,exL,exR,eyStatL,eyStatH);
      grStat->SetMarkerColor(1);      
      grStat->SetMarkerSize(0.6);
      grStat->SetLineColor(1);
      grStat->SetMarkerStyle(21);

      // Add this data set
      vDataTot.push_back(grTot);
      vDataStat.push_back(grStat);
      // Store index in name-to-index array
      vGraphs[dataFile]=vDataTot.size()-1;

      // Create normalized versions (normalized reference)
      for (int i=0; i<200; i++) {
	if (y[i] != 0.0 && i < nlines) {
	  eyStatL[i] /= y[i];
	  eyStatH[i] /= y[i];
	  eyTotL[i]  /= y[i];
	  eyTotH[i]  /= y[i];
	  // Bound the error by 50% to avoid plotting outside plot
	  // (got to be a smarter way of doing this ...)
	  eyStatL[i] = min(Float_t(0.5),eyStatL[i]);
	  eyStatH[i] = min(Float_t(0.5),eyStatH[i]);
	  eyTotL[i]  = min(Float_t(0.5),eyTotL[i]);
	  eyTotH[i]  = min(Float_t(0.5),eyTotH[i]);
	  y[i]       = 1.0;
	} else {
	  y[i] = 1.0;
	  eyStatL[i] = 1.0;
	  eyStatH[i] = 1.0;
	  eyTotL[i] = 1.0;
	  eyTotH[i] = 1.0;
	}
      }   
      TGraphAsymmErrors* grTotN = 
	new TGraphAsymmErrors(nlines,x,y,exL,exR,eyTotL,eyTotH);      
      grTotN->SetFillColor(90);
      TGraphAsymmErrors* grStatN = 
	new TGraphAsymmErrors(nlines,x,y,exL,exR,eyStatL,eyStatH);
      grStatN->SetFillColor(5);

      // Add this normalized data set
      vDataTotN.push_back(grTotN);
      vDataStatN.push_back(grStatN);

      // Add placeholder for theory normalized to data
      TGraphAsymmErrors* grTheoryN = 
	new TGraphAsymmErrors(nlines,x,zero,exL,exR,zero,zero);
      grTheoryN->SetMarkerStyle(20);
      grTheoryN->SetMarkerSize(0.4);
      grTheoryN->SetMarkerColor(1);
      grTheoryN->SetLineColor(1);      
      vTheoryN.push_back(grTheoryN);
      TGraphAsymmErrors* grTheoryNS = 
	new TGraphAsymmErrors(nlines,x,zero,exL,exR,zero,zero);
      grTheoryNS->SetFillStyle(3002);
      grTheoryNS->SetFillColor(3);
      vTheoryNS.push_back(grTheoryNS);

      // Close stream
      in.close();

      // Return succes
      return true;
    }
    
    // Add 1D histogram to Vincia's list
    bool addTH1FPtr(TH1F* histPtrIn, string displayNameIn="none",
		    bool logy=false,bool normalize=true,bool showStatErr=false,
		    string dataFile="none") {
      // Add histogram to specified display, if exists
      vDisplayIndex.push_back(-1);
      bool displayExists = false;
      if (displayNameIn != "none") {
	for (int iDisplay=0; iDisplay<int(displayName.size()); iDisplay++) {
	  if (displayName[iDisplay] == displayNameIn) {
	    vDisplayIndex[vDisplayIndex.size()-1]=iDisplay;
	    displayExists = true;
	    break;
	  }
	}
      }
      vTheory.push_back(histPtrIn);
      vTheoryScale.push_back(1.0);
      // Add local copy that we can modify for plotting without bothering user
      TH1F hIn(*histPtrIn);
      hIn.GetXaxis()->SetLabelFont(42);
      hIn.GetXaxis()->SetLabelSize(0.05);
      hIn.GetXaxis()->SetNdivisions(605);
      hIn.GetXaxis()->SetLabelOffset(0.02);
      hIn.GetYaxis()->SetLabelFont(42);
      hIn.GetYaxis()->SetLabelSize(0.05);
      hIn.GetYaxis()->SetNdivisions(606);
      hIn.GetYaxis()->SetLabelOffset(0.02);
      hIn.SetTitle("");
      if (string(hIn.GetTitle()).find("Mesons") != string::npos || 
	  string(hIn.GetTitle()).find("Baryons") != string::npos) 
	hIn.GetXaxis()->SetLabelSize(0.08);  
      vCopy.push_back(hIn);
      // Add copy for axis only, with zero contents
      TH1F axis(hIn);
      
      axis.Reset();
      axis.SetStats(false);
      axis.SetTitle("");
      axis.SetMinimum(0.5);
      axis.SetMaximum(1.5);
      vAxis.push_back(axis);
      // Set draw option(s)
      vNorm.push_back(normalize);
      vStat.push_back(showStatErr);
      vLogY.push_back(logy);      
      vInitPad.push_back(false);
      // Optionally store name of Data File to compare to
      if (dataFile != "none") {
	// Check if data file exists
	if (!addDataFile(dataFile,normalize)) dataFile="none";
      }
      vData.push_back(dataFile);
      // Always return true
      return true;
    }

    // Update display
    // iDisplayIn
    //    == -1 : update all
    //    >=  0 : update specific display
    bool updateDisplay(int iDisplayIn=-1) {      

      // Do nothing if no histograms
      if (vTheory.size() <= 0) return false;

      // Loop over displays
      for (int iDisplay=0; iDisplay<int(displays.size()); iDisplay++) {

	// If asking for specific display, only update that display
	if (iDisplayIn >= 0 && iDisplay != iDisplayIn) continue;

	// Limit number of plots on each display to finite resolution
	int nPixMin = 5000;
	unsigned int nPlotMax = max(xSiz[iDisplay]*ySiz[iDisplay]/nPixMin,1);
	unsigned int nPlots=0;
	for (int iPlot=0; iPlot<int(vDisplayIndex.size()); iPlot++) {
	  if (vDisplayIndex[iPlot] == iDisplay) nPlots++;
	}
	nPlots = min(nPlotMax,nPlots);	
	// Always at least 1 to avoid divide-by-zero
	if (nPlots <= 1) nPlots = 1;

	// Compute necessary number of rows and columns
	unsigned int nCols = 1;
	unsigned int nRows = 1;
	unsigned int nTot  = 1;
	// Special for 4 plots in 2x2
	if (nPlots == 4) {
	  nCols = 4;
	  nRows = 1;
	  nTot  = 4;
	}
	while (nTot < nPlots) {
	  if (nCols < 3*nRows) {
	    nCols += 1;
	  } else {
	    nRows += 1;
	  }
	  nTot   = nCols * nRows;
	}
	displays[iDisplay]->cd();
	// Check if new division necessary
	if (nRows != nRowsSav[iDisplay] || nCols != nColsSav[iDisplay]) {
	  // Divide and update old
	  displays[iDisplay]->Divide(nCols,nRows);
	  nColsSav[iDisplay] = nCols;
	  nRowsSav[iDisplay] = nRows;	  
	  // Check which ones need to be subdivided into data norm
	  int iPlot=-1;
	  for (long unsigned int iHist=0; iHist<vTheory.size(); iHist++) {
	    if (vDisplayIndex[iHist] != iDisplay) continue;
	    else iPlot++;
	    if (vData[iHist] != "none") {
	      displays[iDisplay]->cd(iPlot+1);
	      gPad->Divide(1,2);
	      gPad->cd(1);
	      gPad->SetBottomMargin(0.1);
	      if (! vInitPad[iHist]) {
		gPad->Draw();
	      } else {
		gPad->Modified();
	      }
	      displays[iDisplay]->cd(iPlot+1);
	      gPad->cd(2);
	      gPad->SetTopMargin(0.00);
	      if (! vInitPad[iHist]) {
		gPad->Draw();
	      } else {
		gPad->Modified();
	      }
	    }
	  }
	}
	// Plot histos
	for (long unsigned int iHist=0; iHist<vTheory.size(); iHist++) {
	  if (vDisplayIndex[iHist] != iDisplay) continue;
	  updatePlot(iHist);
	}
	displays[iDisplay]->cd();
	displays[iDisplay]->Modified();
	displays[iDisplay]->Update();
      }
      return true;
    }

    bool updatePlot(long unsigned int iHist=0) {

      // Sanity check
      if (iHist >= vDisplayIndex.size()) return false;

      // Goto display
      int iDisplay = vDisplayIndex[iHist];
      if (iDisplay == -1) return false;

      // Goto subdivision 
      unsigned int iPlot = -1;
      for (long unsigned int iHistTmp=0; iHistTmp<=iHist; iHistTmp++) {
	if (iHistTmp >= vTheory.size()) break;
	if (vDisplayIndex[iHistTmp] != iDisplay) continue;
	else iPlot++;
      }
      displays[iDisplay]->cd(iPlot+1);      

      // Get name
      string hName(vTheory[iHist]->GetTitle());

      // If data, go to further subdivision
      if (vData[iHist] != "none") {
	gPad->cd(1);
      }

      // If normalizing, create copy to work on
      double sum = 1.0;
      // update copy
      for (int iBin=0; iBin<vTheory[iHist]->GetNbinsX() ; iBin++) {
	vCopy[iHist].SetBinContent(iBin,vTheory[iHist]->GetBinContent(iBin));
	vCopy[iHist].SetBinError(iBin,vTheory[iHist]->GetBinError(iBin));
      }
      vCopy[iHist].SetEntries(vTheory[iHist]->GetEntries());
      vCopy[iHist].SetFillColor(vTheory[iHist]->GetFillColor());
      vCopy[iHist].SetLineColor(8);
      vCopy[iHist].SetMarkerColor(vTheory[iHist]->GetFillColor());

      if (vNorm[iHist]) {
	// Normalize Copy
	sum = vCopy[iHist].GetSumOfWeights()*vCopy[iHist].GetBinWidth(1);
	if (sum == 0.0) sum=1.0;
	vCopy[iHist].Scale(1.0/sum);
      }
      double normFactor=vTheory[iHist]->GetNormFactor();
      if (normFactor > 0.0) {
	vCopy[iHist].Scale(normFactor);
      } else {
	normFactor = 1.0;
      }
      // Set Y max and min values for plot
      double YMax = vTheory[iHist]->GetMaximum();
      YMax = max(YMax,
		 1.05*vCopy[iHist].GetBinContent(vCopy[iHist].GetMaximumBin()));
      vCopy[iHist].SetMaximum(YMax);
      double YMin = vTheory[iHist]->GetMinimum();
      if (vLogY[iHist] && YMin <= 0.0) {
	YMin = max(1e-16,
		   0.95*vCopy[iHist].GetBinContent(vCopy[iHist].GetMinimumBin()));
      }
      vCopy[iHist].SetMinimum(YMin);
      // Draw histogram
      if (! vInitPad[iHist]) {
	if (vLogY[iHist]) gPad->SetLogy();
	vCopy[iHist].Draw("hist"); 
	gPad->Update();
	TPaveStats *st=(TPaveStats*)vCopy[iHist].GetListOfFunctions()->FindObject("stats");
	st->SetTextSizePixels(9);	
	st->SetTextFont(62);
	double nPixPlotX = xSiz[iDisplay]/nColsSav[iDisplay];
	double nPixPlotY = ySiz[iDisplay]/nRowsSav[iDisplay];
	double pSizX = 80.0/nPixPlotX;
	double pSizY = 45.0/nPixPlotY;
	double fSizY = 10./nPixPlotY;
	double fSizX = 11./nPixPlotY;
	if (vData[iHist] != "none") {
	  pSizY *= 2.;
	  fSizY *= 2.;
	  fSizX *= 2.;
	}
	pSizX=1.-min(pSizX,0.6);
	pSizY=1.-min(pSizY,0.6);
	st->SetX1NDC(pSizX);
	st->SetX2NDC(0.99);
	st->SetY2NDC(0.99);
	st->SetY1NDC(pSizY);
	vCopy[iHist].GetXaxis()->SetLabelFont(42);
	vCopy[iHist].GetXaxis()->SetLabelSize(fSizX);
	vCopy[iHist].GetYaxis()->SetLabelSize(fSizY);
	vAxis[iHist].GetXaxis()->SetLabelFont(42);
	vAxis[iHist].GetXaxis()->SetLabelSize(fSizX);
	vAxis[iHist].GetYaxis()->SetLabelSize(fSizY);
	if (string(vTheory[iHist]->GetTitle()).find("Mesons") != string::npos || 
	    string(vTheory[iHist]->GetTitle()).find("Baryons") != string::npos) {
	  vCopy[iHist].GetXaxis()->SetLabelSize(1.5*fSizX);  
	  vAxis[iHist].GetXaxis()->SetLabelSize(1.5*fSizX);  
	  vCopy[iHist].GetXaxis()->SetLabelOffset(0.3*fSizX);  
	  vAxis[iHist].GetXaxis()->SetLabelOffset(0.3*fSizX);  
	}
      } else {
	gPad->Modified();
      }
      if (vStat[iHist]) {
	string opt = "samee0";
	if (! vInitPad[iHist]) {
	  vCopy[iHist].Draw(opt.c_str());  	  	    
	} else {
	  gPad->Modified();
	}
      }      	

      // If Thrust, show dvisions between n-jet regions
      if (hName.find("1-T") != string::npos ) {
	// Draw line
	if ( !vInitPad[iHist] ) {
	  double t45 = 1. - 1./sqrt(3.);
	  double yMin = vTheory[iHist]->GetMinimum();
	  double yMax = vTheory[iHist]->GetMaximum();
	  lineThrust34 = TLine(0.3333,yMin,0.3333,yMax);
	  lineThrust34.SetLineStyle(3);
	  lineThrust34.SetLineColor(1);
	  lineThrust45 = TLine(t45,yMin,t45,yMax);
	  lineThrust45.SetLineStyle(3);
	  lineThrust45.SetLineColor(1);
	  double tLo = vTheory[iHist]->GetBinLowEdge(1);
	  double tHi = tLo + vTheory[iHist]->GetNbinsX() 
	    * vTheory[iHist]->GetBinWidth(1);
	  if (tLo < 0.3333 && tHi > 0.3333) lineThrust34.Draw();
	  if (tLo < t45 && tHi > t45) lineThrust45.Draw();
	}
      }

      // If Heavy Jet Mass, show dvisions between n-jet regions
      else  if (hName.find("Heavy Jet Mass") != string::npos ) {
	// Draw line
	if ( !vInitPad[iHist] ) {
	  double t34 = 1./3.;
	  double yMin = vTheory[iHist]->GetMinimum();
	  double yMax = vTheory[iHist]->GetMaximum();
	  lineMH34 = TLine(t34,yMin,t34,yMax);
	  lineMH34.SetLineStyle(3);
	  lineMH34.SetLineColor(1);
	  double tLo = vTheory[iHist]->GetBinLowEdge(1);
	  double tHi = tLo + vTheory[iHist]->GetNbinsX() 	    
	    * vTheory[iHist]->GetBinWidth(1);
	  if (tLo < t34 && tHi > t34) lineMH34.Draw();
	}
      }

      // If comparing to data, overplot data now
      if (vData[iHist] != "none") {
	// Start in first subdivision: theory vs data
	displays[iDisplay]->cd(iPlot+1);
	gPad->cd(1);	
	int indx = vGraphs[vData[iHist]];
	TGraphAsymmErrors* dataHistTot = vDataTot[indx];	
	if (! vInitPad[iHist]) {
	  dataHistTot->Draw("sameP");
	} else {
	  gPad->Modified();
	}
	// Now go to second subdivision: theory normalized to data
	displays[iDisplay]->cd(iPlot+1);
	gPad->cd(2);	
	if (! vInitPad[iHist]) {
	  vAxis[iHist].Draw();
	  TGraphAsymmErrors* dataTotN = vDataTotN[indx];
	  dataTotN->Draw("samee2");	
	  TGraphAsymmErrors* dataStatN = vDataStatN[indx];
	  dataStatN->Draw("samee2");	
	} else {
	  gPad->Modified();
	}
	// Now normalize theory to data and overplot	
	TGraphAsymmErrors* dataStat = vDataStat[indx];
	TGraphAsymmErrors* theoryN  = vTheoryN[indx];
	TGraphAsymmErrors* theoryNS = vTheoryNS[indx];
	int nPoints = dataStat->GetN();
	for (int iDat=0; iDat<nPoints; iDat++) {
	  Double_t xDat, xLoDat, xHiDat, yDat;
	  dataStat->GetPoint(iDat,xDat,yDat);
	  xLoDat = xDat - dataStat->GetErrorXlow(iDat); 
	  xHiDat = xDat + dataStat->GetErrorXhigh(iDat); 
	  // Loop over theory bins and rebin like data
	  double sumw = 0.0;
	  double sumy = 0.0;
	  double sumErr2 = 0.0;
	  TH1F* h=vTheory[iHist];
	  int nBins = h->GetNbinsX();
	  /*
	  cout << "iDat = "<<iDat <<" xlo = "<<xLoDat<<" xhi = "<<xHiDat<<endl; 
	  */
	  for (int iBin=0; iBin<nBins; iBin++){
	    double xLo = h->GetBinLowEdge(iBin+1);
	    double xHi = xLo + h->GetBinWidth(iBin+1);
	    if (xHi < xLoDat) continue;
	    if (xLo > xHiDat) continue;
	    double bw = xHi-xLo;
	    double w = 1.0;
	    w -= max(0.,xLoDat-xLo)/bw;
	    w -= max(0.,xHi-xHiDat)/bw;
	    w = max(w,0.0);
	    sumw += w;
	    double y    = normFactor*h->GetBinContent(iBin+1)/sum;
	    double yErr = normFactor*h->GetBinError(iBin+1)/sum;	    
	    sumy += w*y;	    
	    sumErr2 += w*pow2(yErr);
	  }	  
	  // Normalize Nch by factor 2 when only even mults
	  if (vData[iHist].find("Nch") != string::npos) sumw /= 2.0;
	  double yTh  = sumy/sumw;	  
	  // At least 5% "systematic" theory error
	  double eyTh  = sqrt(sumErr2)/sumw;
	  double eyThS = sqrt(pow2(2*eyTh) + pow2(0.05*yTh));
	  // If not adding stat, just use 5%
	  if (!vStat[iHist]) {
	    eyTh  = 0.0;
	    eyThS = yTh*0.05;
	  }
	  if (yDat != 0.0) {
	    //	    cout<<"yTh = "<<yTh<<" +/- "<<eyTh<<" yDa = "<<yDat<<endl;
	  }
	  Double_t yThN   = yTh/yDat;
	  Double_t eyThN  = eyTh/yDat;
	  Double_t eyThNS = eyThS/yDat;
	  theoryN->SetPoint(iDat,xDat,yThN);
	  theoryN->SetPointEYhigh(iDat,eyThN);
	  theoryN->SetPointEYlow(iDat,eyThN);
	  theoryNS->SetPoint(iDat,xDat,yThN);
	  theoryNS->SetPointEYhigh(iDat,eyThNS);
	  theoryNS->SetPointEYlow(iDat,eyThNS);
	}
	if (!vInitPad[iHist]) {
	  theoryNS->Draw("samee2");	
	  theoryN->Draw("samePe1");	
	} else {
	  gPad->Modified();
	}

	// If Thrust, show dvisions between 2/3/4-jet regions
	if (hName.find("1-T") != string::npos ) {
	  // Draw line
	  if ( !vInitPad[iHist] ) {
	    double t45 = 1. - 1./sqrt(3.);
	    double yMin = vAxis[iHist].GetMinimum();
	    double yMax = vAxis[iHist].GetMaximum();
	    lineThrust34n = TLine(0.3333,yMin,0.3333,yMax);
	    lineThrust34n.SetLineStyle(3);
	    lineThrust34n.SetLineColor(1);
	    lineThrust45n = TLine(t45,yMin,t45,yMax);
	    lineThrust45n.SetLineStyle(3);
	    lineThrust45n.SetLineColor(1);
	    double tLo = vAxis[iHist].GetBinLowEdge(1);
	    double tHi = tLo + vAxis[iHist].GetNbinsX() 
	      * vAxis[iHist].GetBinWidth(1);
	    if (tLo < 0.3333 && tHi > 0.3333) lineThrust34n.Draw();
	    if (tLo < t45 && tHi > t45) lineThrust45n.Draw();
	  }
	}

	// If Thrust, show dvisions between 2/3/4-jet regions
	else if (hName.find("Heavy Jet Mass") != string::npos ) {
	  // Draw line
	  if ( !vInitPad[iHist] ) {
	    double t34 = 1./3.;
	    double yMin = vAxis[iHist].GetMinimum();
	    double yMax = vAxis[iHist].GetMaximum();
	    lineMH34n = TLine(t34,yMin,t34,yMax);
	    lineMH34n.SetLineStyle(3);
	    lineMH34n.SetLineColor(1);
	    double tLo = vAxis[iHist].GetBinLowEdge(1);
	    double tHi = tLo + vAxis[iHist].GetNbinsX() 
	      * vAxis[iHist].GetBinWidth(1);
	    if (tLo < t34 && tHi > t34) lineMH34n.Draw();
	  }
	}
	
      }
      vInitPad[iHist] = true;

      // Return 
      return true;
    }

    // Save current display
    bool saveDisplay(string displayNameIn, string fileNameIn="eps",
		     string option="") {
      for (int iDisplay=0; iDisplay<int(displayName.size()); iDisplay++) {
	if (displayName[iDisplay] == displayNameIn) {
	  string fName = fileNameIn;
	  if (fileNameIn == "eps") fName=displayNameIn+".eps";
	  if (fileNameIn == "gif") fName=displayNameIn+".gif";
	  if (fileNameIn == "png") fName=displayNameIn+".png";
	  if (fileNameIn == "jpg") fName=displayNameIn+".jpg";
	  if (fileNameIn == "pdf") fName=displayNameIn+".pdf";
	  displays[iDisplay]->SaveAs(fName.c_str(),option.c_str());
	  return true;
	}
      }
      return false;
    }

    // Set verbose level
    void setVerbose(int verboseIn=1) {
      verbose=verboseIn;
    }

  private:
    
    vector<string> displayName;
    vector<TCanvas*> displays;
    bool useDisplay, appIsInit;
    vector<int> xSiz, ySiz, xOffset, yOffset;
    vector<unsigned int> nColsSav, nRowsSav;
    // Vector of Root histrograms to include
    vector<int> vDisplayIndex;
    vector<TH1F*> vTheory;
    vector<TH1F> vCopy, vAxis;
    vector<bool> vNorm, vStat, vLogY, vInitPad;    
    vector<string> vData;
    vector<double> vTheoryScale;
    // Separately indexed data sets, with vData->index map
    map<string,int> vGraphs;
    vector<TGraphAsymmErrors*> vDataStat, vDataTot;
    vector<TGraphAsymmErrors*> vDataStatN, vDataTotN, vTheoryN, vTheoryNS;

    // Lines for dividing thrust
    TLine lineThrust34, lineThrust45, lineThrust34n, lineThrust45n;
    TLine lineMH34, lineMH45, lineMH34n, lineMH45n;

    // Verbose level
    int verbose;

  };
  
}

#endif
